#! /usr/bin/env python
# -*- coding: utf-8 -*-

# import asyncio
# from AiDragonfly import df
# from AiDragonfly import constantModule

# import asyncio
# from . import df
# from . import constantModule

import asyncio
import AiDragonfly.aiqi_main as df
import AiDragonfly.aiqi_model.ble_cc.constantModule as constantModule

_version = 0
def sound(sound): #主控一般音效
    protocol = [0x55, 0x02, 0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
        , 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xAA]
    protocol[4] = sound
    xor = df._data_xor_check(protocol[0:-1])
    protocol[-2] = xor
    loop = asyncio.get_event_loop()
    loop.run_until_complete(df.sendDate(protocol,df.ONEBOT))


class musical_instruments_manage(object): #主控音效管理类
    def _set_data(self,index_number,scale,channel):
        midi_protocol = [0x55, 0x03, 0x01, 0x01, 0x00, 0x00, 0x7F, 0x00, 0x00, 0x00, 0x00
            , 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xAA]
        midi_protocol[4] = index_number
        midi_protocol[5] = scale
        midi_protocol[7] = channel
        xor = df._data_xor_check(midi_protocol[0:-1])
        midi_protocol[-2] = xor
        loop = asyncio.get_event_loop()
        loop.run_until_complete(df.sendDate(midi_protocol, df.ONEBOT))

    def piano(self,midi,channel):
        if channel == 9 or channel > 15 or channel < 0:
            print('Data Range Error')
            print('channel:[0 ~ 8]、[10 ~ 15]')
            return
        self._set_data(constantModule.INSTRUMENTS_PIANO, midi, channel)

    def music_box(self,midi,channel):
        if channel == 9 or channel > 15 or channel < 0:
            print('Data Range Error')
            print('channel:[0 ~ 8]、[10 ~ 15]')
            return
        self._set_data(constantModule.INSTRUMENTS_MUSIC_BOX, midi, channel)

    def bass(self,midi,channel):
        if channel == 9 or channel > 15 or channel<0:
            print('Data Range Error')
            print('channel:[0 ~ 8]、[10 ~ 15]')
            return
        self._set_data(constantModule.INSTRUMENTS_BASS, midi, channel)

    def guitar(self,midi,channel):
        if channel == 9 or channel > 15 or channel < 0:
            print('Data Range Error')
            print('channel:[0 ~ 8]、[10 ~ 15]')
            return
        self._set_data(constantModule.INSTRUMENTS_GUITAR, midi, channel)

    def saxophone(self,midi,channel):
        if channel == 9 or channel > 15 or channel < 0:
            print('Data Range Error')
            print('channel:[0 ~ 8]、[10 ~ 15]')
            return
        self._set_data(constantModule.INSTRUMENTS_SAXOPHONE, midi, channel)

    def flute(self,midi,channel):
        if channel == 9 or channel > 15 or channel < 0:
            print('Data Range Error')
            print('channel:[0 ~ 8]、[10 ~ 15]')
            return
        self._set_data(constantModule.INSTRUMENTS_FLUTE, midi, channel)

    def xylophone(self,midi,channel):
        if channel == 9 or channel > 15 or channel < 0:
            print('Data Range Error')
            print('channel:[0 ~ 8]、[10 ~ 15]')
            return
        self._set_data(constantModule.INSTRUMENTS_XYLOPHONE, midi, channel)

    def violin(self,midi,channel):
        if channel == 9 or channel > 15 or channel < 0:
            print('Data Range Error')
            print('channel:[0 ~ 8]、[10 ~ 15]')
            return
        self._set_data(constantModule.INSTRUMENTS_VIOLIN, midi, channel)

    def harp(self,midi,channel):
        if channel == 9 or channel > 15 or channel < 0:
            print('Data Range Error')
            print('channel:[0 ~ 8]、[10 ~ 15]')
            return
        self._set_data(constantModule.INSTRUMENTS_HARP, midi, channel)

    def percussion(self,percussion):
        self._set_data(constantModule.INSTRUMENTS_PERCUSSION, percussion, 9)


midi = musical_instruments_manage()


def set_volume(volume): #设置主控音量大小
    volume_protocol = [0x55, 0x02, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
        , 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xAA]
    if volume > 12 or volume < 0:
        print('Data Range Error')
        print('volume:[0 ~ 12]')
        return
    volume_protocol[4] = volume
    xor = df._data_xor_check(volume_protocol[0:-1])
    volume_protocol[-2] = xor
    loop = asyncio.get_event_loop()
    loop.run_until_complete(df.sendDate(volume_protocol, df.ONEBOT))


def set_record(record): #设置主控录音
    global _version
    recording_protocol = [0x55, 0x03, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
        , 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xAA]

    if _version >= 517:
        if record == constantModule.RECORD_START:
            print('\033[1;31m Please start recording,supports up to 80 seconds of recording \033[0m')
        elif record == constantModule.RECORD_PLAY:
            print('\033[1;31m The recording starts playing \033[0m')
        elif record == constantModule.RECORD_END:
            print('\033[1;31m End of the recording \033[0m')

        recording_protocol[4] = record
        xor = df._data_xor_check(recording_protocol[0:-1])
        recording_protocol[-2] = xor
        loop = asyncio.get_event_loop()
        loop.run_until_complete(df.sendDate(recording_protocol, df.ONEBOT))
    else:
        print('\033[1;31m Please update the firmware to the latest version \033[0m')






# -*- coding: utf-8 -*-
# @Time    : 2021/8/7 18:15
# @Author  : liumingming
# @FileName: aiqi_bluetooth_uuid.py
# @Company ：http://www.iqi-inc.com/


xor_index = 18

def _data_xor_check(data):  # 异或校验函数 主要用于报文的校验
    value = 0x00
    for i in data:
        value = value ^ i
    return value



class BlueReport:

    create_connect = bytearray([0x57, 0x01, 0x02, 0xAA])

    def __init__(self):
        self._onebotcc_register_start = bytearray([0x55, 0x01, 0x03, 0x00, 0x01, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                 0x00, 0x00, 0xAA])
        self._onebotcc_register_stop = bytearray(
            [0x55, 0x01, 0x03, 0x01, 0x01, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
             0x00, 0x00, 0xAA])

    @property
    def onebotcc_register_start(self):
        self._onebotcc_register_start[xor_index] = 0
        self._onebotcc_register_start[xor_index]=_data_xor_check(self._onebotcc_register_start[0:-1])
        return self._onebotcc_register_start

    @property
    def onebotcc_register_stop(self):
        self._onebotcc_register_stop[xor_index] = 0
        self._onebotcc_register_stop[xor_index] = _data_xor_check(self._onebotcc_register_stop[0:-1])
        return self._onebotcc_register_stop






bluereport = BlueReport()






#! /usr/bin/env python
# -*- coding: utf-8 -*-
import atexit
import threading

import AiDragonfly
from bleak.backends.device import BLEDevice
from bleak.backends.scanner import AdvertisementData

from AiDragonfly.aiqi_model.bluetooth import aiqi_bluetooth
from AiDragonfly.comm_protocol.aiqi_blue_report import bluereport
from AiDragonfly.comm_protocol.aiqi_mesh_report import mesh_report, _mesh_login_class
from AiDragonfly.common.onebot_common_function import Aiqi_Print_HEX
from AiDragonfly.aiqi_model.ble_cc import constantModule, motorModule, musicModule, sensorModule
import asyncio
import time
import sys

# import aiqi_model

# The global variable
import AiDragonfly.aiqi_bluetooth_device_type as aiqi_blue_type
from AiDragonfly.Get_Mesh_Devices_Message import Get_Onebot_Device_Message_Init
import AiDragonfly.aiqi_bluetooth_save_json as aiqi_file

_volume = 0
_engine = 0
_voltage = 0
_regeditNum = 0
_regedit = [[0 for i in range(100)] for j in range(25)]
__dev_type = 0

blue_scan_parm = None
onebot_client = None

mesh_control_flag = 0
mesh_mode_flag = 0

mesh_config_ack = False

__onebot_login = _mesh_login_class()
__aiqi_adapter = aiqi_bluetooth.Aiqi_Bluetooth_Class()


def _protocol_analysis(num, data):  # 数据接收分析函数，根据报文协议 解析收到的数据
    global _regeditNum, _regedit, _volume, _engine, _voltage
    global i, __onebot_login
    global mesh_config_ack
    global mesh_control_flag, mesh_mode_flag
    length = num
    _receive_message = data
    # Aiqi_Print_HEX(data)
    if length > 3:  # 接收到的数据个数 小于三个 则抛弃
        if _receive_message[0] == 0x55:  # 红蓝主控的报文分析 0x55是红蓝主控协议的报头
            buf = _receive_message
            if buf[0] == 0x55 and buf[-1] == 0xAA:  # 协议报头 报尾
                if buf[1] == 0x01 and buf[2] == 0x04 and buf[3] == 0x01 and buf[4] == 0x00:  # 根据具体协议解析即可
                    _volume = buf[5]
                    _engine = buf[6]
                    # print('音量',buf)
                elif buf[1] == 0x02 and buf[2] == 0x06 and buf[3] == 0x00 and buf[4] == 0xC8:
                    volume = buf[8] * 256 + buf[9]
                    # print('Synchronous voltage')
                    # print('主控电压',buf)
                    pass
                elif buf[1] == 0x01 and buf[2] == 0x02 and buf[3] == 0x00 and buf[4] == 0x00:
                    # print('The heartbeat packets')
                    pass
                elif buf[1] == 0x01 and buf[2] == 0x03 and buf[3] == 0x00 and buf[4] == 0x01:
                    # print('The registry starts uploading')
                    pass
                elif buf[1] == 0x01 and buf[2] == 0x03 and buf[3] == 0x01 and buf[4] == 0x01:
                    # print('Registry upload complete')
                    pass
                elif buf[1] == 0x01 and buf[2] == 0x07 and buf[3] == 0x00:
                    # print('SN',buf)
                    pass
                elif buf[1] == 0x01 and buf[2] == 0x06 and buf[3] == 0x00:
                    musicModule._version = (
                                                   buf[4] << 24) + (buf[5] << 16) + (buf[6] << 8) + buf[7]
                    # print('固件版本号',buf)
                elif buf[1] == 0x01 and buf[2] == 0x06 and buf[3] == 0x01:
                    # print('语音版本号',buf)
                    pass
                elif buf[1] == 0x05 and buf[2] == 0x00 and buf[3] == 0x00:  # 注册信息处理
                    if _regeditNum == 0:  # 注册个数为零 存储第一条注册信息
                        # print('有新传感器,类型：', buf[5],'。  注册号：',buf[4],'已注册个数：',_regeditNum+1)
                        # print('数据：',buf)
                        _regedit[_regeditNum][0:20] = buf[0:20]
                        _regeditNum = _regeditNum + 1
                    else:
                        while True:
                            # 接收到注册报文后，先判断注册表中是否已存在此注册信息
                            for i in list(range(_regeditNum)):
                                tmp1 = buf[6:11]
                                tmp2 = bytearray(_regedit[i][6:11])
                                if tmp1 == tmp2:
                                    break
                                else:
                                    i = i + 1
                            if i == _regeditNum:  # 注册表中没有此条注册信息，填入注册表
                                # print('有新传感器,类型：', buf[5],'。  注册号：',buf[4],'已注册个数：',_regeditNum+1)
                                # print('数据：',buf)
                                _regedit[_regeditNum][0:20] = buf[0:20]
                                _regeditNum = _regeditNum + 1
                                break
                            else:
                                break
                # 以上主要为主控连接成功后的 初始交互信息和注册表
                # 下面主要是接收到传感器数据的处理
                elif buf[1] == 0x02 and buf[2] == 0x02 and buf[3] == 0x05:
                    motorModule.motor._rotation_completed_flag = 1
                    motorModule.motor._rotation_completed_port = buf[4]
                elif buf[1] == 0x02 and buf[2] == 0x06 and buf[3] == 0x00 and buf[4] == 0x0B:
                    if buf[11] == 4 or buf[11] == 5 or buf[11] == 6 or buf[11] == 7:
                        motorModule.motor._port_speed = buf[5] * 256 + buf[6]
                    elif buf[11] == 0 or buf[11] == 1 or buf[11] == 2 or buf[11] == 3:
                        motorModule.motor._port_angle = buf[5] * 256 + buf[6]

                elif buf[1] == 0x02 and buf[2] == 0x06 and buf[3] == 0x00:
                    if buf[10] == 0x00:  # 传感器状态在线
                        if buf[4] == sensorModule._sensor.regedit_num:  # 首先判断传感器注册号是否匹配
                            for i in list(range(_regeditNum)):
                                # 注册号匹配
                                if sensorModule._sensor.regedit_num == _regedit[i][4]:
                                    if _regedit[i][5] == constantModule.SENSOR_DIGITAL_SERVO:  # 舵机
                                        if buf[11] == constantModule.DIGITALSERVO_GETANGLE:
                                            motorModule.digitalServo._angle = buf[5] * \
                                                                              256 + buf[6]

                                    # 设备类型匹配,处理数据.这个是多功能颜色传感器
                                    elif _regedit[i][5] == constantModule.SENSOR_OPTICAL:
                                        if buf[11] == constantModule.OPTICAL_COLOR:  # 颜色
                                            sensorModule.sensor_optical._color = buf[6]
                                        elif buf[11] == constantModule.OPTICAL_GRAY_SCALE:  # 灰度
                                            sensorModule.sensor_optical._grayScale = buf[6]
                                        elif buf[11] == constantModule.OPTICAL_RGB:  # RGB
                                            sensorModule.sensor_optical._R = buf[5]
                                            sensorModule.sensor_optical._G = buf[6]
                                            sensorModule.sensor_optical._B = buf[7]
                                        # 环境光
                                        elif buf[11] == constantModule.OPTICAL_AMBIENT_LIGHT_INTENSITY:
                                            sensorModule.sensor_optical._ambientLightIntensity = buf[
                                                                                                     5] * 256 + buf[6]
                                        elif buf[11] == constantModule.OPTICAL_KEY_STATUS:
                                            sensorModule.sensor_optical._keyStatus = buf[6]
                                        # elif buf[11] == constantModule.OPTICAL_SIGNAL_STRENGTH:
                                        #     sensorModule.sensor_optical._signalStrength = buf[6]

                                    # 光敏传感器
                                    elif _regedit[i][5] == constantModule.SENSOR_PHOTOSENSITIVE:
                                        if buf[11] == constantModule.PHOTOSENSITIVE_AMBIENT_LIGHT_INTENSITY:
                                            sensorModule.sensor_photosensitive._ambientLightIntensity = buf[5] * 256 + \
                                                                                                        buf[6]

                                    elif _regedit[i][5] == constantModule.SENSOR_LASER:  # 激光传感器
                                        if buf[11] == constantModule.LASER_DISTANCE_MM:
                                            sensorModule.sensor_laser._distance_mm = buf[5] * \
                                                                                     256 + buf[6]
                                        elif buf[11] == constantModule.LASER_DISTANCE_CM:
                                            sensorModule.sensor_laser._distance_cm = buf[6]
                                        elif buf[11] == constantModule.LASER_KEY_STATUS:
                                            sensorModule.sensor_laser._keyStatus = buf[6]
                                        # elif buf[11] == constantModule.LASER_SIGNAL_STRENGTH:
                                        #     sensorModule.sensor_laser._signal_strength = buf[6]

                                    # elif _regedit[i][5] == constantModule.SENSOR_FLAME:
                                    #     if buf[11] == constantModule.FLAME_DIRECTION:
                                    #         sensorModule.sensor_infraredRadar._flameDirection = buf[6]
                                    #     elif buf[11] == constantModule.FLAME_INTENSITY:
                                    #         sensorModule.sensor_infraredRadar._flameIntensity = buf[6]
                                    #     elif buf[11] == constantModule.FLAME_KEY_STATUS:
                                    #         sensorModule.sensor_infraredRadar._keyStatus = buf[6]
                                    #     elif buf[11] == constantModule.FLAME_FOOTBALL_DIRECTION:
                                    #         sensorModule.sensor_infraredRadar._footballDirection = buf[6]

                                    elif _regedit[i][5] == constantModule.SENSOR_GEOMAGNETIC:  # 地磁传感器
                                        if buf[11] == constantModule.GEOMAGNETIC_FIELD_ANGLE:
                                            sensorModule.sensor_geomagnetic._magneticFieldAngle = buf[
                                                                                                      5] * 256 + buf[6]
                                        elif buf[11] == constantModule.GEOMAGNETIC_KEY_STATUS:
                                            sensorModule.sensor_geomagnetic._keyStatus = buf[6]

                                    elif _regedit[i][5] == constantModule.SENSOR_ATTITUDE:  # 六轴传感器
                                        if buf[11] == constantModule.ATTITUDE_DICE:
                                            sensorModule.sensor_attitude._dice = buf[6]
                                        # elif buf[11] == constantModule.ATTITUDE_STEPS:
                                        #     sensorModule.sensor_attitude._steps = buf[5]*256 + buf[6]
                                        # elif buf[11] == constantModule.ATTITUDE_SIGNAL_STRENGTH:
                                        #     sensorModule.sensor_attitude._signalStrength = buf[6]
                                        elif buf[11] == constantModule.ATTITUDE_KEY_STATUS:
                                            sensorModule.sensor_attitude._keyStatus = buf[6]
                                        elif buf[11] == constantModule.ATTITUDE_ACCELERATION_X:
                                            sensorModule.sensor_attitude._acceleration_X = buf[
                                                                                               5] * 256 + buf[6]
                                        elif buf[11] == constantModule.ATTITUDE_ACCELERATION_Y:
                                            sensorModule.sensor_attitude._acceleration_Y = buf[
                                                                                               5] * 256 + buf[6]
                                        elif buf[11] == constantModule.ATTITUDE_ACCELERATION_Z:
                                            sensorModule.sensor_attitude._acceleration_Z = buf[
                                                                                               5] * 256 + buf[6]
                                        elif buf[11] == constantModule.ATTITUDE_ANGULAR_VELOCITY_X:
                                            sensorModule.sensor_attitude._angularVelocity_X = buf[
                                                                                                  5] * 256 + buf[6]
                                        elif buf[11] == constantModule.ATTITUDE_ANGULAR_VELOCITY_Y:
                                            sensorModule.sensor_attitude._angularVelocity_Y = buf[
                                                                                                  5] * 256 + buf[6]
                                        elif buf[11] == constantModule.ATTITUDE_ANGULAR_VELOCITY_Z:
                                            sensorModule.sensor_attitude._angularVelocity_Z = buf[
                                                                                                  5] * 256 + buf[6]
                                        # elif buf[11] == constantModule.ATTITUDE_INCLINATION_ANGLE_X:
                                        #     sensorModule.sensor_attitude._inclinationAngle_X = buf[6]
                                        # elif buf[11] == constantModule.ATTITUDE_INCLINATION_ANGLE_Y:
                                        #     sensorModule.sensor_attitude._inclinationAngle_Y = buf[6]
                                        # elif buf[11] == constantModule.ATTITUDE_INCLINATION_ANGLE_Z:
                                        #     sensorModule.sensor_attitude._inclinationAngle_Z = buf[6]

                                    else:
                                        print('Sensor not found')
                    else:
                        print('sensor offline')
        elif _receive_message[0] == 0x01:  # mesh设备的处理报文
            buf = _receive_message
            if buf[1] == 0x79:  # 配置回应
                if buf[2] == 0:
                    mesh_config_ack = True
                elif buf[2] == 1:
                    mesh_config_ack = False
                else:
                    mesh_config_ack = False
            elif buf[1] == 0x53:  # 接收到随机值
                mesh_random = buf[2]
                __onebot_login.dev_random = buf[2]

                mesh_sn_flag = 0
            elif buf[1] == 0x47:  # 成功登录
                if buf[2] == 1:
                    __onebot_login.dev_login_stats = True
                else:
                    __onebot_login.dev_login_stats = False
            elif buf[1] == 0x81:  # 平衡机器人模式
                if buf[2] == 0:
                    mesh_mode_flag = 0
                elif buf[2] == 1:
                    mesh_mode_flag = 1
                elif buf[3] == 2:
                    mesh_mode_flag = 2
                elif buf[4] == 3:
                    mesh_mode_flag = 3

        elif _receive_message[0] == 0x04:  # mesh设备的处理报文  接收到版本号
            buf = _receive_message
            if buf[1] == 0x4C:
                v1 = buf[2] << 24
                v2 = buf[3] << 16
                v3 = buf[4] << 8
                v4 = buf[5]
                mesh_version = v1 + v2 + v3 + v4
                # print('version ' + str(mesh_version))
                __onebot_login.dev_version = mesh_version

        elif _receive_message[0] == 0x07:  # mesh设备的处理报文 接收到SN
            buf = _receive_message
            if buf[1] == 0x4E:
                __onebot_login.dev_sn = buf[2:9]


__dev_mac_addr = ''

__onebot_local_save_list = []

__get_message_done_stats = False


def __get_mssage_callback(dev_list):
    global __get_message_done_stats, __onebot_local_save_list

    __onebot_local_save_list = []
    __onebot_local_save_list = dev_list[:]
    __get_message_done_stats = True


__bluetooth_scan_list = []
__wait_char_stats = False
__print_char = ""
__print_char_num = 0
__print_char_num_max = 0


def __print_wait_char(time=0.1, maxnum=10):
    global __print_char, __print_char_num, __wait_char_stats
    __print_char_num_max = maxnum
    if __wait_char_stats:
        __print_char += '█'
        print("\r" + __print_char + "  " + str(__print_char_num) + "/" + str(int(maxnum)), end='')
        threading.Timer(time, __print_wait_char, args=(time, maxnum)).start()
    __print_char_num += 1
    if __print_char_num > __print_char_num_max:
        __print_char_num = 0
        __print_char = ""
        __wait_char_stats = False


def __bluetooth_scan_callback(device: BLEDevice, advertisement_data: AdvertisementData):
    global __bluetooth_scan_list

    if device.name != '' and device.name is not None:
        if len(__bluetooth_scan_list) != 0:
            for item in __bluetooth_scan_list:
                if item.address == device.address:
                    return
        __bluetooth_scan_list.append(device)
        # print('bluetooth_scan_callback' + device.name + ' ' + device.address, "RSSI:", device.rssi)


def __user_console_select(devs):
    user_select_buff = devs[:]
    user_done = False
    devs_index = []
    user_new_buff = []
    for item in devs:
        devs_index.append(item.index)

    while not user_done:
        def checkvalid(check_buff):
            for i in check_buff:
                if i not in devs_index:
                    print(i)
                    print(devs_index)
                    return False
            return True

        aiqi_blue_type.Print_Scan_Onebot_Device(user_select_buff)
        print(
            'Please enter the number of the connected index , multiple sequences separated by spaces,Or directly enter, all the default configuration')
        print('Input sequence, determine the control sequence, please choose according to your needs')
        user_input = input('(Enter or input index):').split()
        if len(user_input) != 0:
            user_input = list(map(int, user_input))
            if checkvalid(user_input):
                user_new_buff = []
                for i in user_input:
                    for item in user_select_buff:
                        if item.index == i:
                            user_new_buff.append(item)
                user_done = True
            else:
                print("Input error, please try again")
        else:
            user_new_buff = user_select_buff[:]
            user_done = True
    return user_new_buff


__App_Reset = ''


async def onebot_getdevices_process():
    global __dev_type, __dev_mac_addr, __wait_char_stats

    global __bluetooth_scan_list, __get_message_done_stats

    local_save_list = []
    scan_stats = True
    if __App_Reset == 'new':
        aiqi_file.delete_local_save_device()
    else:
        local_save_list = aiqi_file.Read_Local_Onebot_Device()
    if len(local_save_list) == 0:
        print('No local storage device found')
    else:
        scan_stats = False
        __dev_mac_addr = local_save_list[0].devid
        __onebot_local_save_list = local_save_list[:]
        __dev_type = aiqi_blue_type.Read_Onebot_Device_type(__onebot_local_save_list)
    if scan_stats:
        print('Start scanning')
        __bluetooth_scan_list = []
        __aiqi_adapter.set_scan_callback_regiser(__bluetooth_scan_callback)

        await __aiqi_adapter.scan_ctrl(True)
        __wait_char_stats = True
        __print_wait_char(0.5, blue_scan_parm.scantimeout / 0.5)

        await asyncio.sleep(blue_scan_parm.scantimeout + 1)
        await __aiqi_adapter.scan_ctrl(False)
        __wait_char_stats = False
        print('\nscan stop')

        devices = __bluetooth_scan_list[:]
        devices = __aiqi_adapter.scan_devices_operation(
            devices, blue_scan_parm.scannamelist, blue_scan_parm.scanmaclist)
        __onebot_local_save_list = aiqi_blue_type.select_onebot_devices(
            devices, scan_name=blue_scan_parm.scannamelist)
        if len(devices) == 0:
            print(
                "\n" + 'Please Turn on the master motor power switch or Please insert the Bluetooth adapter')
            sys.exit('aiqi quit')

        __dev_type = aiqi_blue_type.Read_Onebot_Device_type(__onebot_local_save_list)
        print('Getting information......')
        if __dev_type == aiqi_blue_type.aiqi_device_type_class.Aiqi_Onebot_Mesh_Dev:
            await Get_Onebot_Device_Message_Init(__onebot_local_save_list, __get_mssage_callback)
            while __get_message_done_stats is False:
                pass
                time.sleep(0.1)  # 等待获取完成
            __get_message_done_stats = False
            __onebot_local_save_list = __user_console_select(__onebot_local_save_list)[:]
        aiqi_file.Save_Local_Onebot_Device(__onebot_local_save_list)
        print("Successfully saved")
        # 获取设备信息
    print('The local device information is as follows:   ')
    aiqi_file.print_Local_Onebot_Device()
    time.sleep(1)  # 等待断开
    __dev_mac_addr = __onebot_local_save_list[0].devid


'''
onebot login
'''


async def __onebot_login_process():
    global __wait_char_stats
    try:
        print("Please wait for a connection, up to eight seconds......")
        __wait_char_stats = True
        __print_wait_char(1, 8)
        await __aiqi_adapter.connect(__dev_mac_addr, timeout=8)

    except:
        __wait_char_stats = False
        print("\nDevice connection timeout")
        sys.exit()
    __wait_char_stats = False
    print("\n")

    __aiqi_adapter.set_disconnect_callback_register(_disconnect_callback)
    __aiqi_adapter.set_receive_callback_register(_protocol_analysis)
    global onebot_client, ONEBOT_Connect_Stats

    if __dev_type == 1 or __dev_type == 4:  # 蓝主控
        print('shake .....')
        # 发送连接报文
        __aiqi_adapter.set_receive_callback_register(_protocol_analysis)
        await __aiqi_adapter.sendData(bluereport.create_connect)
        # await client.write_gatt_char(aiqi_uuid.Aiqi_Bluetooth_Rx_Char_Uuid, bluereport.create_connect)
        # await client.start_notify(aiqi_uuid.Aiqi_Bluetooth_Tx_Char_Uuid,
        #                           _protocol_analysis)  # 打开蓝牙设备的发送，接收数据都在_protocol_analysis函数中
        await asyncio.sleep(3)
        # await client.stop_notify(aiqi_uuid.Aiqi_Bluetooth_Tx_Char_Uuid)  # 停止蓝牙设备的发送

    elif __dev_type == 2 or __dev_type == 3:  # 红主控 or 越野车 同上
        print('shake .....')


    elif __dev_type == 5:  #
        print('Detection version.....', end='')
        await __aiqi_adapter.senddata(mesh_report.get_version)
        time.sleep(0.1)
        print(' ' + __onebot_login.Get_Version_Str())
        if __onebot_login.dev_version >= __onebot_login.need_login_version:
            await __aiqi_adapter.senddata(mesh_report.get_sn)
            time.sleep(0.1)
            if __onebot_login.dev_sn is not None:
                await __aiqi_adapter.senddata(mesh_report.get_random)
                await asyncio.sleep(0.05)
                time.sleep(0.1)
                mesh_report.mesh_sn = []
                mesh_report.mesh_sn = __onebot_login.dev_sn[:]
                mesh_report.mesh_random = __onebot_login.dev_random
                await __aiqi_adapter.senddata(mesh_report.login_in)
                await asyncio.sleep(0.05)
                time.sleep(0.1)
                if __onebot_login.dev_login_stats:
                    print('login ok')
                else:
                    print('login err')
                    sys.exit('login err')
            else:
                print('sn err')
                sys.exit('sn err')
        elif __onebot_login.dev_version == 0:
            print('versin err')
            sys.exit('err')
        else:
            print('free login')

    else:
        print('Device type does not exist:', __dev_type)
        sys.exit()


def Aiqi_Bluetooth_Send(data, log=False):
    if log:
        Aiqi_Print_HEX(data)
    if __onebot_login.dev_login_stats:
        data.append(__onebot_login.dev_random)
    if __aiqi_adapter.connect_stats is True:
        loop = asyncio.get_event_loop()
        loop.run_until_complete(sendDate(data))  # 配置设备 发送模式配置信息
    else:
        print('ble is disconnect')


def __Aiqi_Bluetooth_Retry_Connect():
    print('Aiqi_Bluetooth_Retry_Connect')


# 蓝牙发送数据
async def sendDate(data):
    databytes = bytearray(data)
    if __aiqi_adapter.connect_stats:
        await __aiqi_adapter.senddata(databytes)
    else:
        print('ble disconect')


User_Disconnect_Handle = None


def _disconnect_callback(res):
    global ONEBOT_Connect_Stats, onebot_client
    # print('disconnect')
    ONEBOT_Connect_Stats = False
    onebot_client = None
    __onebot_login.Clear()
    if User_Disconnect_Handle is not None:
        User_Disconnect_Handle(res)


# def Aiqi_Bluetooth_Disconnect_CallBack_Register(callback):
#     global User_Disconnect_Handle
#     # print('connect callback reg')
#     User_Disconnect_Handle = callback


ONEBOT_Connect_Stats = False


async def app_exit():
    await __aiqi_adapter.disconnect()
    time.sleep(0.1)


@atexit.register
def atexit_fun():
    print('sys exit')
    loop = asyncio.get_event_loop()
    loop.run_until_complete(app_exit())


'''
app_reset : if is 'new' ,will clear local save all devices

'''


def start(app_reset='', scantimeout=5, rssi=-100, namelist=[], maclist=[],disconnect_callback=None):
    global blue_scan_parm, __App_Reset,User_Disconnect_Handle
    __App_Reset = app_reset.lower()
    print('start')
    parm = aiqi_blue_type.aiqi_bluetooth_scan_class()
    parm.scantimeout = scantimeout
    parm.scannamelist = namelist[:]
    parm.scanmaclist = maclist[:]
    User_Disconnect_Handle = disconnect_callback
    blue_scan_parm = parm
    loop = asyncio.get_event_loop()
    loop.run_until_complete(onebot_getdevices_process())

    loop = asyncio.get_event_loop()
    loop.run_until_complete(__onebot_login_process())

    return onebot_client



# -*- coding: utf-8 -*-
# @Time    : 2021/8/9 10:23
# @Version : V1.0.0
# @Author  : liumingming
# @FileName: aiqi_bluetooth.py
# @Version : V1.0.0
# @Company ：http://www.iqi-inc.com/
import asyncio

from bleak import BleakClient
from bleak import BleakScanner


class Aiqi_Bluetooth_bleak_Class(BleakClient):
     async def __aexit__(self, exc_type, exc_val, exc_tb):
         pass


# -*- coding: utf-8 -*-
# @Time    : 2021/8/9 10:23
# @Version : V1.0.0
# @Author  : liumingming
# @FileName: aiqi_bluetooth.py
# @Version : V1.0.0
# @Company ：http://www.iqi-inc.com/
import asyncio
import time

from bleak import BleakClient
from bleak import BleakScanner

from AiDragonfly.aiqi_model.bluetooth.aiqi_bluetooth_bleak import Aiqi_Bluetooth_bleak_Class
from AiDragonfly.common.onebot_common_function import Aiqi_Print_HEX
from bleak.backends.device import BLEDevice
from bleak.backends.scanner import AdvertisementData
from AiDragonfly.aiqi_model.bluetooth.aiqi_bluetooth_uuid import aiqi_bluetooth_uuid_class as aiqi_uuid


class Aiqi_Bluetooth_Class:
    client_log = False
    scan_log = False
    bluetooth_client = None
    bluetooth_scaner = None
    connect_stats = False
    __user_scaner_callback = None
    __user_disconnect_callback = None
    __user_recive_callback = None
    __send_err_cnt = 0
    __log_title = 'aiqi blue driver '
    __user_scan_list = []

    def __init__(self, clientlog=False,scanlog=False):
        self.client_log = clientlog
        self.scan_log = scanlog
        self.bluetooth_scaner = BleakScanner()
        self.bluetooth_client = None
        self.bluetooth_scaner.register_detection_callback(self._scan_callback)

    def _scan_callback(self,device: BLEDevice, advertisement_data: AdvertisementData):
        if self.scan_log:
            if device.name != '' and device.name is not None:
                if len(self.__user_scan_list) != 0:
                    for item in self.__user_scan_list:
                        if item.address == device.address:
                            return
                self.__user_scan_list.append(device)
                print(self.__log_title+' scancallback '+device.name + ' ' + device.address, "RSSI:", device.rssi)
        if self.__user_scaner_callback is not None:
            self.__user_scaner_callback(device,advertisement_data)


    def set_scan_callback_regiser(self, callback):
        self.__user_scaner_callback = callback

    def set_disconnect_callback_register(self, callback):
        self.__user_disconnect_callback = callback
    def set_receive_callback_register(self, callback):
        self.__user_recive_callback = callback

    @property
    def Get_User_Scan_List(self):
        return self.__user_scan_list

    def scan_devices_operation(self,devices, namelist=[], maclist=[]):
        newdevs = []
        if len(namelist) == 0 and len(maclist) == 0:
            return devices
        if len(maclist) == 0:
            for dev in devices:
                if dev.name in namelist:
                    # print('has1 :' + dev.name)
                    newdevs.append(dev)
        else:
            for dev in devices:
                if dev.address in maclist:
                    # print('has2 :' + dev.name+' mac:'+dev.address)
                    newdevs.append(dev)
        return newdevs

    async def scan_ctrl(self, ctrl):
        if ctrl:
            self.__user_scan_list = []
            await self.bluetooth_scaner.start()
        else:
            await self.bluetooth_scaner.stop()

    async def connect(self, mac, timeout=8):
        async with Aiqi_Bluetooth_bleak_Class(mac, timeout=timeout) as client:
            def _disconnect_callback(res):
                if self.__user_disconnect_callback is not None:
                    self.__user_disconnect_callback(res)

            def _client_recive_callback(len, data):
                if self.client_log:
                    Aiqi_Print_HEX(data, self.__log_title + ' rev:')
                if self.__user_recive_callback is not None:
                    self.__user_recive_callback(len, data)
            await asyncio.sleep(0.1)
            # client.stop_notify()
            # client.disconnect()
            if self.client_log:
                print(self.__log_title + ' connect ok')
            self.connect_stats = True
            self.bluetooth_client = client
            client.set_disconnected_callback(_disconnect_callback)
            await client.start_notify(aiqi_uuid.Aiqi_Bluetooth_Tx_Char_Uuid, _client_recive_callback)
            return client
    async def senddata(self,data):
        try:
            await self.bluetooth_client.write_gatt_char(aiqi_uuid.Aiqi_Bluetooth_Rx_Char_Uuid, data)
            await asyncio.sleep(0.1)
            self.__send_err_cnt = 0
        except:
            print(self.__log_title + 'send err')
            self.connect_stats = False
            await self.bluetooth_client.disconnect()
    async def disconnect(self):
        if self.bluetooth_client is not None:
            self.bluetooth_client.set_disconnected_callback(None)
            await self.bluetooth_client.disconnect()
            await asyncio.sleep(0.1)
            self.bluetooth_client = None




import sys

Onebot_Dev_Name_Items = ['Onebot', 'Mibot', 'Mi_Builder_C1B', 'Onebot_Edu_01B', "OneBot_XL_M",
                         "OneBot_St_M", "OneBot_LED", "OneBot_BL_M"]
Onebot_Dev_MeshName_Items = ["OneBot_XL_M", "OneBot_St_M", "OneBot_LED", "OneBot_BL_M"]
Onebot_Dev_BleName_Items = ['Onebot', 'Mibot', 'Mi_Builder_C1B', 'Onebot_Edu_01B']

class aiqi_device_type_class:
    Aiqi_Null_Dev = 0
    Aiqi_Onebot_Dev = 1
    Aiqi_Mibot_Dev = 2
    Aiqi_Mi_Builder_C1B_Dev = 3
    Aiqi_Onebot_Edu_01B_Dev = 4
    Aiqi_Onebot_Mesh_Dev = 5

class aiqi_bluetooth_dev_mes_class:
    index = 0
    name = ''
    devid = ''
    mesh_addr = None
    rssi = 0
    version = None

    def __init__(self, index=0, name=None, id=None, rssi=None):
        self.index = index
        self.name = name
        self.devid = id
        self.rssi = rssi

    def Set_Message(self, index,name, id, rssi, addr, version):
        self.index = index
        self.name = name
        self.devid = id
        self.rssi = rssi
        self.mesh_addr = addr
        self.version = version

    def print_title(self):
        print("index  name  deviceid  rssi")

    def __str__(self):
        restr= ('[ ' +'index:' + str(self.index) + ' name: ' + str(self.name) + ' devid: ' + str(self.devid) + ' addr: ' + str
        (self.mesh_addr) + ' rssi: ' + str(self.rssi) + ']')

        return restr


class aiqi_bluetooth_scan_class:
    scantimeout = 0
    scannamelist = []
    scanmaclist = []
    scanrssi = 0

    def __init__(self, scantimeout=3, scannamelist=[], scanmaclist=[],rssi =-100):
        self.scantimeout = scantimeout
        self.scannamelist = scannamelist
        self.scanmaclist = scanmaclist
        self.scanrssi = rssi


def select_onebot_devices(devices, rssi=-100, scan_name=[], scan_mac=[]):
    result_buff = []
    index = 0
    for dev in devices:
        if dev.name in Onebot_Dev_Name_Items and dev.rssi > rssi:

            dev_item = aiqi_bluetooth_dev_mes_class()
            dev_item.index = index
            dev_item.name = dev.name
            dev_item.devid = dev.address
            dev_item.rssi = dev.rssi
            index += 1
            if len(scan_name) == 0 and len(scan_mac) == 0:  # 不筛选
                result_buff.append(dev_item)
                # print('find ' + dev.name + " " + dev.address + ' ' + str(dev.rssi))
            elif len(scan_name) > 0 and len(scan_mac) == 0:  # 只筛选名字
                for name in scan_name:
                    if name == dev_item.name:
                        result_buff.append(dev_item)
                        # print('find ' + dev.name + " " + dev.address + ' ' + str(dev.rssi))
            elif len(scan_name) > 0 and len(scan_mac) > 0:  # 筛选名字和mac的设备
                for name in scan_name:
                    if name == dev_item.name:
                        for mac in scan_mac:
                            if mac == dev_item.devid:
                                result_buff.append(dev_item)
                                # print('find ' + dev.name + " " + dev.address + ' ' + str(dev.rssi))

    return result_buff


'''
mesh 设备判断
禁止ble 与 mesh 设备混合使用
'''


def IsMesh_Devices(buff):
    stats = False
    dev_len = len(buff)
    search = 0
    for item in buff:
        if item.name in Onebot_Dev_MeshName_Items:
            stats = True
            search +=1
        else:
            stats = False
    if stats == True and  search <dev_len:
        sys.exit('禁止普通设备与mesh 设备混合使用')
    return stats


'''

'''


def Read_Onebot_Device_type(devs):
    dev_type = aiqi_device_type_class.Aiqi_Null_Dev
    if len(devs) == 0:
        return []
    if IsMesh_Devices(devs) :
        dev_type = aiqi_device_type_class.Aiqi_Onebot_Mesh_Dev
    else:# 不是mesh device,取第一个判断类型
        _name = devs[0].name
        if _name == 'Onebot':
            dev_type=aiqi_device_type_class.Aiqi_Onebot_Dev
        elif _name == 'Mibot':
            dev_type = aiqi_device_type_class.Aiqi_Mibot_Dev
        elif _name == 'Mi_Builder_C1B':
            dev_type = aiqi_device_type_class.Aiqi_Mi_Builder_C1B_Dev
        elif _name == 'Onebot_Edu_01B':
            dev_type = aiqi_device_type_class.Aiqi_Onebot_Edu_01B_Dev
        else:
            pass
            sys.exit('不支持的设备名称，程序即将退出')

    return dev_type

def Print_Scan_Onebot_Device(devbuf):
    print('***********************local device list***********************')
    for item in devbuf:
        print(item)
    print('***************************************************************')







# -*- coding: utf-8 -*-
# @Time    : 2021/8/7 18:15
# @Author  : liumingming
# @FileName: aiqi_bluetooth_uuid.py
# @Company ：http://www.iqi-inc.com/



class aiqi_bluetooth_uuid_class:
    Aiqi_Bluetooth_Services_Uuid = '6e400001-b5a3-f393-e0a9-e50e24dcca9e'
    Aiqi_Bluetooth_Rx_Char_Uuid = '6e400002-b5a3-f393-e0a9-e50e24dcca9e'
    Aiqi_Bluetooth_Tx_Char_Uuid = '6e400003-b5a3-f393-e0a9-e50e24dcca9e'


# -*- coding: utf-8 -*-
# @Time    : 2021/8/9 9:04
# @Author  : liumingming
# @FileName: __init__.py.py
# @Company ：http://www.iqi-inc.com/

import platform
import sys

from AiDragonfly.aiqi_main import start,Aiqi_Bluetooth_Send

import AiDragonfly.aiqi_model as model


__run_system = platform.system()

if __run_system == "Darwin":
    pass
elif __run_system == "Windows":
    platform.release()
    if platform.release() >= '10':
        win_ver = platform.version()
        win_verlist = win_ver.split('.')
        __ver_h = int(win_verlist[0])
        __ver_M = int(win_verlist[1])
        __ver_L = int(win_verlist[2])

        if __ver_h >= 10 and __ver_M >=0 and __ver_L >= 16299:
            pass
        else:
            print(__run_system + " " + platform.release() + ' ' + platform.version())
            print('System version is too low, please update windows 10.0.16299 above')
            sys.exit(0)

    else:
        print(__run_system+" "+platform.release()+' '+platform.version())
        print('The system does not support')
        sys.exit(0)
    pass
else:
    print(__run_system + " " + platform.release() + ' ' + platform.version())
    print('The system does not support')
    sys.exit(0)




#! /usr/bin/env python
# -*- coding: utf-8 -*-

# from AiDragonfly import constantModule
# from AiDragonfly import df
# from AiDragonfly import sensorModule

# from . import constantModule
# from . import df
# from . import sensorModule
# import asyncio

import AiDragonfly.aiqi_model.ble_cc.constantModule as constantModule
import AiDragonfly.aiqi_main as df
import AiDragonfly.aiqi_model.ble_cc.sensorModule as  sensorModule
import asyncio


class Motor_manage(object): #电机管理类
    def __init__(self):
        self._port_speed = 0
        self._port_angle = 0
        self._rotation_completed_flag = 0
        self._rotation_completed_port = 0

    def _get_data(self,subclass): #获取数据
        protocol = [0x55, 0x02, 0x06, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
            , 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xAA]

        protocol[4] = 0x0B
        protocol[5] = subclass
        xor = df._data_xor_check(protocol[0:-1])
        protocol[-2] = xor
        loop = asyncio.get_event_loop()
        loop.run_until_complete(df.sendDate(protocol, df.ONEBOT))
        loop = asyncio.get_event_loop()
        loop.run_until_complete(df.receiveData(df.ONEBOT))

    def set_speed_portA(self,speed): #设置端口A速度
        protocol = [0x55, 0x02, 0x02, 0x00, 0x80, 0x00, 0x80, 0x00, 0x00, 0x80, 0x80
            , 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xAA]
        if speed > 127 or speed < -127:
            print('Data Range Error')
            print('speed:[-127 ~ 127]')
            return
        if speed == 0:
            speed = speed + 128
        elif speed < 0:
            speed = 128 - abs(speed)
            if speed > 127:
                speed = 127
        elif speed > 0:
            speed = speed + 128
            if speed > 255:
                speed = 255
        protocol[4] = speed
        xor = df._data_xor_check(protocol[0:-1])
        protocol[-2] = xor
        loop = asyncio.get_event_loop()
        loop.run_until_complete(df.sendDate(protocol, df.ONEBOT))

    def set_speed_portB(self,speed):
        protocol = [0x55, 0x02, 0x02, 0x00, 0x80, 0x00, 0x80, 0x00, 0x00, 0x80, 0x80
            , 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xAA]
        if speed > 127 or speed < -127:
            print('Data Range Error')
            print('speed:[-127 ~ 127]')
            return
        if speed == 0:
            speed = speed + 128
        elif speed < 0:
            speed = 128 - abs(speed)
            if speed > 127:
                speed = 127
        elif speed > 0:
            speed = speed + 128
            if speed > 255:
                speed = 255
        protocol[6] = speed
        xor = df._data_xor_check(protocol[0:-1])
        protocol[-2] = xor
        loop = asyncio.get_event_loop()
        loop.run_until_complete(df.sendDate(protocol, df.ONEBOT))

    def set_speed_portC(self,speed):
        protocol = [0x55, 0x02, 0x02, 0x00, 0x80, 0x00, 0x80, 0x00, 0x00, 0x80, 0x80
            , 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xAA]
        if speed > 127 or speed < -127:
            print('Data Range Error')
            print('speed:[-127 ~ 127]')
            return
        if speed == 0:
            speed = speed + 128
        elif speed < 0:
            speed = 128 - abs(speed)
            if speed > 127:
                speed = 127
        elif speed > 0:
            speed = speed + 128
            if speed > 255:
                speed = 255
        protocol[9] = speed
        xor = df._data_xor_check(protocol[0:-1])
        protocol[-2] = xor
        loop = asyncio.get_event_loop()
        loop.run_until_complete(df.sendDate(protocol, df.ONEBOT))

    def set_speed_portD(self,speed):
        protocol = [0x55, 0x02, 0x02, 0x00, 0x80, 0x00, 0x80, 0x00, 0x00, 0x80, 0x80
            , 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xAA]
        if speed > 127 or speed < -127:
            print('Data Range Error')
            print('speed:[-127 ~ 127]')
            return
        if speed == 0:
            speed = speed + 128
        elif speed < 0:
            speed = 128 - abs(speed)
            if speed > 127:
                speed = 127
        elif speed > 0:
            speed = speed + 128
            if speed > 255:
                speed = 255
        protocol[10] = speed
        xor = df._data_xor_check(protocol[0:-1])
        protocol[-2] = xor
        loop = asyncio.get_event_loop()
        loop.run_until_complete(df.sendDate(protocol, df.ONEBOT))

    def set_speed(self,portA_speed,portB_speed,portC_speed,portD_speed): #同时设置四个端口速度
        protocol = [0x55, 0x02, 0x02, 0x00, 0x80, 0x00, 0x80, 0x00, 0x00, 0x80, 0x80
            , 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xAA]
        if portA_speed > 127 or portA_speed < -127:
            print('Data Range Error')
            print('portA_speed:[-127 ~ 127]')
            return
        if portB_speed > 127 or portB_speed < -127:
            print('Data Range Error')
            print('portB_speed:[-127 ~ 127]')
            return
        if portC_speed > 127 or portC_speed < -127:
            print('Data Range Error')
            print('portC_speed:[-127 ~ 127]')
            return
        if portD_speed > 127 or portD_speed < -127:
            print('Data Range Error')
            print('portD_speed:[-127 ~ 127]')
            return

        if portA_speed == 0:
            portA_speed = portA_speed + 128
        elif portA_speed < 0:
            portA_speed = 128 - abs(portA_speed)
            if portA_speed > 127:
                portA_speed = 127
        elif portA_speed > 0:
            portA_speed = portA_speed + 128
            if portA_speed > 255:
                portA_speed = 255

        if portB_speed == 0:
            portB_speed = portB_speed + 128
        elif portB_speed < 0:
            portB_speed = 128 - abs(portB_speed)
            if portB_speed > 127:
                portB_speed = 127
        elif portB_speed > 0:
            portB_speed = portB_speed + 128
            if portB_speed > 255:
                portB_speed = 255

        if portC_speed == 0:
            portC_speed = portC_speed + 128
        elif portC_speed < 0:
            portC_speed = 128 - abs(portC_speed)
            if portC_speed > 127:
                portC_speed = 127
        elif portC_speed > 0:
            portC_speed = portC_speed + 128
            if portC_speed > 255:
                portC_speed = 255

        if portD_speed == 0:
            portD_speed = portD_speed + 128
        elif portD_speed < 0:
            portD_speed = 128 - abs(portD_speed)
            if portD_speed > 127:
                portD_speed = 127
        elif portD_speed > 0:
            portD_speed = portD_speed + 128
            if portD_speed > 255:
                portD_speed = 255

        protocol[4] = portA_speed
        protocol[6] = portB_speed
        protocol[9] = portC_speed
        protocol[10] = portD_speed
        xor = df._data_xor_check(protocol[0:-1])
        protocol[-2] = xor
        loop = asyncio.get_event_loop()
        loop.run_until_complete(df.sendDate(protocol, df.ONEBOT))

    def set_angle(self,motor_port,speed,angle): #设置角度
        protocol = [0x55, 0x02, 0x02, 0x04, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00
            , 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xAA]
        motor_port = motor_port + 1
        if motor_port > 4:
            print('Data Range Error')
            print('port:[1 ~ 4]')
            print('speed:[-127 ~ 127]')
            print('angle:[0 ~ 65535]')
            return
        if speed > 127 or speed < -127:
            print('Data Range Error')
            print('port:[1 ~ 4]')
            print('speed:[-127 ~ 127]')
            print('angle:[0 ~ 65535]')
            return
        if angle > 65535 or angle < 0:
            print('Data Range Error')
            print('port:[1 ~ 4]')
            print('speed:[-127 ~ 127]')
            print('angle:[0 ~ 65535]')
            return

        if speed == 0:
            speed = speed + 128
        elif speed < 0:
            speed = 128 - abs(speed)
            if speed > 127:
                speed = 127
        elif speed > 0:
            speed = speed + 128
            if speed > 255:
                speed = 255

        anglel = angle % 256
        angleh = angle >> 8

        protocol[4] = motor_port
        protocol[5] = speed
        protocol[6] = angleh
        protocol[7] = anglel
        xor = df._data_xor_check(protocol[0:-1])
        protocol[-2] = xor
        loop = asyncio.get_event_loop()
        loop.run_until_complete(df.sendDate(protocol, df.ONEBOT))

    def get_speed(self,motor_port): #读取电机端口速度值
        motor_port = motor_port + 4
        if motor_port > 7 or motor_port < 4:
            print('Data Range Error')
            print('speedport:[4 ~ 7]')
            return
        self._get_data(motor_port)
        return df._hex2int(self._port_speed, 16)

    def get_angle(self, motor_port): #获取端口角度
        if motor_port < 0 or motor_port > 3:
            print('Data Range Error')
            print('angleport:[0 ~ 3]')
            return
        self._get_data(motor_port)
        return df._hex2int(self._port_angle, 16)

    def whichPort_rotationCompleted(self): #判断哪个端口电机转动完成
        loop = asyncio.get_event_loop()
        loop.run_until_complete(df.receiveData(df.ONEBOT))
        if self._rotation_completed_flag:
            self._rotation_completed_flag = 0
            if self._rotation_completed_port == 0x01:
                self._rotation_completed_port = 'portA_completed'
            elif self._rotation_completed_port == 0x02:
                self._rotation_completed_port = 'portB_completed'
            elif self._rotation_completed_port == 0x03:
                self._rotation_completed_port = 'portC_completed'
            elif self._rotation_completed_port == 0x04:
                self._rotation_completed_port = 'portD_completed'

            return self._rotation_completed_port


motor = Motor_manage()


class DigitalServo_manage(object): #舵机管理类
    def __init__(self):
        self._angle = 0

    def set_angle(self,angle,*,ID=1): #设置舵机角度
        if angle > 160 or angle < -160:
            print('Data Range Error')
            print('angle:[-160 ~ 160]')
            return
        angle = df._int2hex(angle,16)
        anglel = angle % 256
        angleh = angle >> 8
        sensorModule._sensor.set_sensor(constantModule.SENSOR_DIGITAL_SERVO, constantModule.DIGITALSERVO_SETANGLE, angleh, anglel, 0, ID=ID)

    def set_power(self,power,*,ID=1): #设置舵机功率
        if power > 100 or power < -100:
            print('Data Range Error')
            print('power:[-100 ~ 100]')
            return
        power = df._int2hex(power,8)
        sensorModule._sensor.set_sensor(constantModule.SENSOR_DIGITAL_SERVO, constantModule.DIGITALSERVO_SETPOWER, 0, power, 0, ID=ID)

    def get_angle(self,*,ID=1): #获取舵机角度
        sensorModule._sensor.get_sensor_data(constantModule.SENSOR_DIGITAL_SERVO, constantModule.DIGITALSERVO_GETANGLE, ID=ID)
        return df._hex2int(self._angle,16)


digitalServo = DigitalServo_manage()






# -*- coding: utf-8 -*-
# @Time    : 2021/8/7 18:15
# @Author  : liumingming
# @FileName: aiqi_bluetooth_uuid.py
# @Company ：http://www.iqi-inc.com/


from AiDragonfly.common.onebot_common_function import onebot_xor_check

class _mesh_login_class:
    dev_sn = None
    dev_random = 0
    dev_login_stats = False
    dev_version = 0
    need_login_version = 0x040000

    def Get_Sn_Xor(self):
        pass

    def Get_Version_Str(self):
        return str(self.dev_version >> 24) + '.' + str((self.dev_version >> 16) & 0xff) + '.' + str(
            (self.dev_version >> 8) & 0xff) + '.' + str(self.dev_version & 0xff)

    def Clear(self):
        self.dev_sn = None
        self.dev_random = 0
        self.dev_login_stats = False
        self.dev_version = None

class MeshReport:
    mesh_sn = []
    mesh_sn_xor = 0
    mesh_random = 0

    get_version = bytearray([0x01, 0x54, 0x00])
    get_sn = bytearray([0x00, 0x56])
    get_random = bytearray([0x00, 0x52])


    def __init__(self):
        self._login_in = bytearray([1,0x46,0,0])

    @property
    def login_in(self):
        # self._login_in = bytearray([0x01, 0x46,self.mesh_sn_xor, self.mesh_random])
        self.mesh_sn_xor = onebot_xor_check(self.mesh_sn)
        self._login_in[0] = 0x01
        self._login_in[1] = 0x46
        self._login_in[2] = self.mesh_sn_xor
        self._login_in[3] = self.mesh_random
        return self._login_in


mesh_report = MeshReport()



import os

import json

from AiDragonfly.aiqi_bluetooth_device_type import aiqi_bluetooth_dev_mes_class

Save_Json_path = os.path.expanduser('~')
Save_Json_path = os.path.join(Save_Json_path, 'aiqi_python_temp')
save_json_name = 'aiqi_save_devices_message.json'


def _write_local_json(json_data):
    json_path = os.path.join(Save_Json_path, save_json_name)
    json_result = None
    if not os.path.exists(Save_Json_path):
        os.mkdir(Save_Json_path)
    file = open(json_path, 'w')
    file.write(json_data)
    file.close()


def _read_local_json(name):
    json_path = os.path.join(Save_Json_path, name)

    if not os.path.isfile(json_path):
        print('file not exist,create new file')
        file = open(json_path, 'w')
        file.write('')
        file.close()
        return None
    file = open(json_path)
    json_class = None
    try:
        json_class = json.load(file)
        # print(json_class)
    except:
        print('json load except')
        file = open(json_path, 'w')
        file.write('')
    file.close()
    return json_class


def delete_local_save_device():
    _write_local_json('')




def Save_Local_Onebot_Device(dev_buff):
    save_buf = []
    for item in dev_buff:
        user_item = {'index':item.index, 'name': item.name, 'devid': item.devid, 'mesh_addr': item.mesh_addr, 'rssi': item.rssi,
                     'version': item.version}
        save_buf.append(user_item)
    json_data = json.dumps(save_buf, indent=4, separators=(',', ': '))
    _write_local_json(json_data)


def Read_Local_Onebot_Device():
    result = []
    read_buf = _read_local_json(save_json_name)
    # print('Read_Local_Onebot_Device:'+json.dumps(read_buf))
    for item in read_buf:
        dev = aiqi_bluetooth_dev_mes_class()
        dev.Set_Message(item['index'],item['name'], item['devid'], item['rssi'], item['mesh_addr'], item['version'])
        result.append(dev)
    return result


def print_Local_Onebot_Device():
    print('***********************local device list***********************')
    dev_item = Read_Local_Onebot_Device()
    for item in dev_item:
        print(item)

    print('*****************************************************************')



#! /usr/bin/env python
# -*- coding: utf-8 -*-

# from AiDragonfly import df
# from AiDragonfly import constantModule
# import asyncio

# import asyncio
# from . import df
# from . import constantModule

import asyncio
import AiDragonfly.aiqi_main as df
import AiDragonfly.aiqi_model.ble_cc.constantModule as constantModule


class _sensor_manage(object): #传感器管理类
    def __init__(self):
        self.color = 0
        self.sensorID = 0
        self.cnt = 0
        self.index = 0
        self.regedit_num = 0

    def set_sensor(self,sensor_class, subclass, data1, data2, data3, *, ID=1): #向传感器发送控制数据
        protocol = [0x55, 0x02, 0x06, 0x03, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
            , 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xAA]
        self.cnt = 0
        self.index = 0
        self.sensorID = 0
        for self.cnt in list(range(df._regeditNum)): #从注册列表中查找是否存在此传感器
            self.index = self.index + 1
            if sensor_class == df._regedit[self.cnt][5]: #传感器类型匹配
                self.sensorID = self.sensorID + 1
                if self.sensorID < ID:
                    continue
                elif self.sensorID == ID: #找到指定的传感器后,填写协议内容
                    protocol[5] = df._regedit[self.cnt][4]
                    protocol[6] = subclass
                    protocol[7] = data1
                    protocol[8] = data2
                    protocol[9] = data3
                    xor = df._data_xor_check(protocol[0:-1])
                    protocol[-2] = xor
                    loop = asyncio.get_event_loop()
                    loop.run_until_complete(df.sendDate(protocol, df.ONEBOT)) #发送
                    break

    def get_sensor_data(self,sensor_class, subclass, *, ID=1): #获取传感器数据
        protocol = [0x55, 0x02, 0x06, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
            , 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xAA]
        self.cnt = 0
        self.index = 0
        self.sensorID = 0
        self.regedit_num = 0
        for self.cnt in list(range(df._regeditNum)): #在注册表中查找是否有次传感器
            self.index = self.index + 1
            if sensor_class == df._regedit[self.cnt][5]:
                self.sensorID = self.sensorID + 1
                if self.sensorID < ID:
                    continue
                elif self.sensorID == ID:
                    self.regedit_num = df._regedit[self.cnt][4]
                    protocol[4] = self.regedit_num
                    protocol[5] = subclass
                    xor = df._data_xor_check(protocol[0:-1])
                    protocol[-2] = xor
                    loop = asyncio.get_event_loop()
                    loop.run_until_complete(df.sendDate(protocol, df.ONEBOT))
                    loop = asyncio.get_event_loop()
                    loop.run_until_complete(df.receiveData(df.ONEBOT))
                    break


_sensor = _sensor_manage()


class Optical(object): #多功能颜色传感器
    def __init__(self):
        self._color = 0
        self._grayScale = 0
        self._R = 0
        self._G = 0
        self._B = 0
        self._ambientLightIntensity = 0
        self._keyStatus = 0
        self._signalStrength = 0

    def get_color(self,*,ID=1): #获取颜色
        _sensor.get_sensor_data(constantModule.SENSOR_OPTICAL, constantModule.OPTICAL_COLOR, ID=ID)
        if self._color == 1:
            self._color = 'blue'
        elif self._color == 2:
            self._color = 'red'
        elif self._color == 3:
            self._color = 'yellow'
        elif self._color == 4:
            self._color = 'white'
        elif self._color == 5:
            self._color = 'green'
        elif self._color == 6:
            self._color = 'black'
        elif self._color == 8:
            self._color = 'grey'
        elif self._color == 12:
            self._color = 'purple'
        else:
            self._color = 'none'
        return self._color


    def get_grayScale(self,*,ID=1): #获取灰度值
        _sensor.get_sensor_data(constantModule.SENSOR_OPTICAL, constantModule.OPTICAL_GRAY_SCALE, ID=ID)
        return self._grayScale

    def get_RGB(self,*,ID=1): #获取RGB值
        _sensor.get_sensor_data(constantModule.SENSOR_OPTICAL, constantModule.OPTICAL_RGB, ID=ID)
        return  self._R,self._G,self._B

    def get_ambientLightIntensity(self,*,ID=1):
        _sensor.get_sensor_data(constantModule.SENSOR_OPTICAL, constantModule.OPTICAL_AMBIENT_LIGHT_INTENSITY, ID=ID)
        return self._ambientLightIntensity

    def get_keyStatus(self,*,ID=1):
        _sensor.get_sensor_data(constantModule.SENSOR_OPTICAL, constantModule.OPTICAL_KEY_STATUS, ID=ID)
        if self._keyStatus == 0:
            self._keyStatus = 'release'
        elif self._keyStatus == 1:
            self._keyStatus = 'press'
        return self._keyStatus

    # def get_signalStrength(self,*,ID=1):
    #     _sensor.get_sensor_data(constantModule.SENSOR_OPTICAL,constantModule.OPTICAL_SIGNAL_STRENGTH,ID=ID)
    #     return self._signalStrength

    def set_lightUp(self,R,G,B,*,ID=1):
        if R > 255 or R < 0:
            print('Data Range Error')
            print('R:[0 ~ 255]')
            print('G:[0 ~ 255]')
            print('B:[0 ~ 255]')
            return
        if G > 255 or G < 0:
            print('Data Range Error')
            print('R:[0 ~ 255]')
            print('G:[0 ~ 255]')
            print('B:[0 ~ 255]')
            return
        if B > 255 or B < 0:
            print('Data Range Error')
            print('R:[0 ~ 255]')
            print('G:[0 ~ 255]')
            print('B:[0 ~ 255]')
            return
        _sensor.set_sensor(constantModule.SENSOR_OPTICAL, constantModule.OPTICAL_LIGHT_UP, R, G, B, ID=ID)


sensor_optical = Optical()


class Laser(object): #激光传感器
    def __init__(self):
        self._distance_mm = 0
        self._distance_cm = 0
        self._keyStatus = 0
        self._signalStrength = 0

    def get_distance_mm(self,*,ID=1):
        _sensor.get_sensor_data(constantModule.SENSOR_LASER, constantModule.LASER_DISTANCE_MM, ID=ID)
        return self._distance_mm

    def get_distance_cm(self,*,ID=1):
        _sensor.get_sensor_data(constantModule.SENSOR_LASER, constantModule.LASER_DISTANCE_CM, ID=ID)
        return self._distance_cm

    def get_keyStatus(self,*,ID=1):
        _sensor.get_sensor_data(constantModule.SENSOR_LASER, constantModule.LASER_KEY_STATUS, ID=ID)
        if self._keyStatus == 0:
            self._keyStatus = 'release'
        elif self._keyStatus == 1:
            self._keyStatus = 'press'
        return self._keyStatus

    # def get_signalStrength(self,*,ID=1):
    #     _sensor.get_sensor_data(constantModule.SENSOR_LASER,constantModule.LASER_SIGNAL_STRENGTH,ID=ID)
    #     return self._signalStrength


sensor_laser = Laser()


# class InfraredRadar(object):
#     def __init__(self):
#         self._flameDirection = 0
#         self._flameIntensity = 0
#         self._keyStatus = 0
#         self._footballDirection = 0
#
#     def get_flameDirection(self,*,ID=1):
#         _sensor.get_sensor_data(constantModule.SENSOR_FLAME,constantModule.FLAME_DIRECTION,ID=ID)
#         if self._flameDirection == 1:
#             self._flameDirection = 'second from the left'
#         elif self._flameDirection == 2:
#             self._flameDirection = 'first from the left'
#         elif self._flameDirection == 3:
#             self._flameDirection = 'middle'
#         elif self._flameDirection == 4:
#             self._flameDirection = 'first from the right'
#         elif self._flameDirection == 5:
#             self._flameDirection = 'second from the right'
#         else:
#             self._flameDirection = 'none'
#         return self._flameDirection
#
#     def get_flameIntensity(self,*,ID=1):
#         _sensor.get_sensor_data(constantModule.SENSOR_FLAME,constantModule.FLAME_INTENSITY,ID=ID)
#         return self._flameIntensity
#
#     def get_keyStatus(self,*,ID=1):
#         _sensor.get_sensor_data(constantModule.SENSOR_FLAME,constantModule.FLAME_KEY_STATUS,ID=ID)
#         if self._keyStatus == 0:
#             self._keyStatus = 'release'
#         elif self._keyStatus == 1:
#             self._keyStatus = 'press'
#         return self._keyStatus
#
#     def get_footballDirection(self,*,ID=1):
#         _sensor.get_sensor_data(constantModule.SENSOR_FLAME,constantModule.FLAME_FOOTBALL_DIRECTION,ID=ID)
#         if self._footballDirection == 1:
#             self._footballDirection = 'second from the left'
#         elif self._footballDirection == 2:
#             self._footballDirection = 'first from the left'
#         elif self._footballDirection == 3:
#             self._footballDirection = 'middle'
#         elif self._footballDirection == 4:
#             self._footballDirection = 'first from the right'
#         elif self._footballDirection == 5:
#             self._footballDirection = 'second from the right'
#         else:
#             self._footballDirection = 'none'
#         return self._footballDirection
#
#
# sensor_infraredRadar = InfraredRadar()


class Geomagnetic(object): #地磁传感器
    def __init__(self):
        self._magneticFieldAngle = 0
        self._keyStatus = 0

    def get_magneticFieldAngle(self,*,ID=1):
        _sensor.get_sensor_data(constantModule.SENSOR_GEOMAGNETIC, constantModule.GEOMAGNETIC_FIELD_ANGLE, ID=ID)
        return self._magneticFieldAngle

    def get_keyStatus(self,*,ID=1):
        _sensor.get_sensor_data(constantModule.SENSOR_GEOMAGNETIC, constantModule.GEOMAGNETIC_KEY_STATUS, ID=ID)
        if self._keyStatus == 0:
            self._keyStatus = 'release'
        elif self._keyStatus == 1:
            self._keyStatus = 'press'
        return self._keyStatus


sensor_geomagnetic = Geomagnetic()


class Attitude(object): #六轴传感器
    def __init__(self):
        self._dice = 0
        self._keyStatus = 0
        self._acceleration_X = 0
        self._acceleration_Y = 0
        self._acceleration_Z = 0
        self._angularVelocity_X = 0
        self._angularVelocity_Y = 0
        self._angularVelocity_Z = 0
        self._inclinationAngle_X = 0
        self._inclinationAngle_Y = 0
        self._inclinationAngle_Z = 0
        self._attitude_inclinationAngle_Z = 0
        self._steps = 0
        self._signalStrength = 0

    def get_surface(self,*,ID=1):
        _sensor.get_sensor_data(constantModule.SENSOR_ATTITUDE, constantModule.ATTITUDE_DICE, ID=ID)
        return self._dice

    def get_keyStatus(self,*,ID=1):
        _sensor.get_sensor_data(constantModule.SENSOR_ATTITUDE, constantModule.ATTITUDE_KEY_STATUS, ID=ID)
        if self._keyStatus == 0:
            self._keyStatus = 'release'
        elif self._keyStatus == 1:
            self._keyStatus = 'press'
        return self._keyStatus

    def get_acceleration_X(self,*,ID=1):
        _sensor.get_sensor_data(constantModule.SENSOR_ATTITUDE, constantModule.ATTITUDE_ACCELERATION_X, ID=ID)
        return self._acceleration_X

    def get_acceleration_Y(self,*,ID=1):
        _sensor.get_sensor_data(constantModule.SENSOR_ATTITUDE, constantModule.ATTITUDE_ACCELERATION_Y, ID=ID)
        return self._acceleration_Y

    def get_acceleration_Z(self,*,ID=1):
        _sensor.get_sensor_data(constantModule.SENSOR_ATTITUDE, constantModule.ATTITUDE_ACCELERATION_Z, ID=ID)
        return self._acceleration_Z

    def get_angularVelocity_X(self,*,ID=1):
        _sensor.get_sensor_data(constantModule.SENSOR_ATTITUDE, constantModule.ATTITUDE_ANGULAR_VELOCITY_X, ID=ID)
        return self._angularVelocity_X

    def get_angularVelocity_Y(self,*,ID=1):
        _sensor.get_sensor_data(constantModule.SENSOR_ATTITUDE, constantModule.ATTITUDE_ANGULAR_VELOCITY_Y, ID=ID)
        return self._angularVelocity_Y

    def get_angularVelocity_Z(self,*,ID=1):
        _sensor.get_sensor_data(constantModule.SENSOR_ATTITUDE, constantModule.ATTITUDE_ANGULAR_VELOCITY_Z, ID=ID)
        return self._angularVelocity_Z

    # def get_inclinationAngle_X(self,*,ID=1):
    #     _sensor.get_sensor_data(constantModule.SENSOR_ATTITUDE,constantModule.ATTITUDE_INCLINATION_ANGLE_X,ID=ID)
    #     return self._inclinationAngle_X
    #
    # def get_inclinationAngle_Y(self,*,ID=1):
    #     _sensor.get_sensor_data(constantModule.SENSOR_ATTITUDE,constantModule.ATTITUDE_INCLINATION_ANGLE_Y,ID=ID)
    #     return self._inclinationAngle_Y
    #
    # def get_inclinationAngle_Z(self,*,ID=1):
    #     _sensor.get_sensor_data(constantModule.SENSOR_ATTITUDE,constantModule.ATTITUDE_INCLINATION_ANGLE_Z,ID=ID)
    #     return self._inclinationAngle_Z
    #
    # def get_Steps(self,*,ID=1):
    #     _sensor.get_sensor_data(constantModule.SENSOR_ATTITUDE,constantModule.ATTITUDE_STEPS,ID=ID)
    #     return self._steps

    # def get_signalStrength(self,*,ID=1):
    #     _sensor.get_sensor_data(constantModule.SENSOR_ATTITUDE,constantModule.ATTITUDE_SIGNAL_STRENGTH,ID=ID)
    #     return self._signalStrength


sensor_attitude = Attitude()


class Photosensitive(object): #开发板 光敏控制函数
    def __init__(self):
        self._ambientLightIntensity = 0

    def get_photosensitiveValue(self,*,ID=1):
        _sensor.get_sensor_data(constantModule.SENSOR_PHOTOSENSITIVE,
                                constantModule.PHOTOSENSITIVE_AMBIENT_LIGHT_INTENSITY, ID=ID)
        return self._ambientLightIntensity


sensor_photosensitive = Photosensitive()


class RGBLight(object): #开发板 RGB灯控制函数
    def set_color(self,R,G,B,*,ID=1):
        if R > 255 or R < 0:
            print('Data Range Error')
            print('R:[0 ~ 255]')
            print('G:[0 ~ 255]')
            print('B:[0 ~ 255]')
            return
        if G > 255 or G < 0:
            print('Data Range Error')
            print('R:[0 ~ 255]')
            print('G:[0 ~ 255]')
            print('B:[0 ~ 255]')
            return
        if B > 255 or B < 0:
            print('Data Range Error')
            print('R:[0 ~ 255]')
            print('G:[0 ~ 255]')
            print('B:[0 ~ 255]')
            return
        _sensor.set_sensor(constantModule.SENSOR_RGBLIGHT, constantModule.RGBLIGHT_CONTROL, R, G, B, ID=ID)


sensor_RGBLight = RGBLight()


class Buzzer(object): #开发板蜂鸣器控制函数
    def set_audio(self,buzzer_tone,buzzer_note,*,ID=1):
        if buzzer_tone > 3 or buzzer_tone < 0:
            print('Data Range Error')
            print('BUZZER_TONE_HIGH、BUZZER_TONE_MIDDLE、BUZZER_TONE_LOW')
            return
        if buzzer_note > 7 or buzzer_note < 1:
            print('Data Range Error')
            print('BUZZER_NOTE_DO、BUZZER_NOTE_RE、BUZZER_NOTE_MI、BUZZER_NOTE_FA、BUZZER_NOTE_SO、BUZZER_NOTE_LA、BUZZER_NOTE_SI')
        data = buzzer_tone*10 + buzzer_note
        _sensor.set_sensor(constantModule.SENSOR_BUZZER, constantModule.BUZZER_VOLUME, 0, data, 0, ID=ID)

    def set_music(self,music,*,ID=1):
        _sensor.set_sensor(constantModule.SENSOR_BUZZER, constantModule.BUZZER_MUSIC, 0, 0, music, ID=ID)


sensor_buzzer = Buzzer()


def find_sensor(sensor, *, ID=1): #查找传感器 当多个传感器时 可以使用此函数确认对应关系
    _protocol = [0x55, 0x02, 0x06, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
        , 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xAA]
    _sensorID = 0
    _cnt = 0
    _index = 0
    _regedit_num = 0
    _optical_cnt = 0
    _laser_cnt = 0
    _servo_cnt = 0
    _geomagnetic_cnt = 0
    _flame_cnt = 0
    _attitude_cnt = 0
    _rgblight_cnt = 0
    _buzzer_cnt = 0
    _sensitive_cnt = 0
    _stsensor = ''

    print('\033[1;31m The list of sensors is as follows: \033[0m')
    print("+" + "-" * 11 + "+" + "-" * 15 + "+")
    print("|" + " " * 2 + "Sensor" + " " * 3 + "|" + " " + "Serial number" + " " + "|")
    print("+" + "-" * 11 + "+" + "-" * 15 + "+")
    for _cnt in list(range(df._regeditNum)): #把注册表中的传感器打印出来
        if df._regedit[_cnt][5] == constantModule.SENSOR_OPTICAL: #多功能颜色传感器
            _optical_cnt = _optical_cnt + 1
            print("|" + " " * 2 + "OPTICAL" + " " * 2 + "|" + " " * 6, _optical_cnt, " " * 6 + "|")
            print("+" + "-" * 11 + "+" + "-" * 15 + "+")
            if sensor == constantModule.SENSOR_OPTICAL:
                _stsensor = 'OPTICAL'
        elif df._regedit[_cnt][5] == constantModule.SENSOR_DIGITAL_SERVO:
            _servo_cnt = _servo_cnt + 1
            print("|" + " " * 2 + "SERVO" + " " * 4 + "|" + " " * 6, _servo_cnt, " " * 6 + "|")
            print("+" + "-" * 11 + "+" + "-" * 15 + "+")
            if sensor == constantModule.SENSOR_DIGITAL_SERVO:
                _stsensor = 'SERVO'
        elif df._regedit[_cnt][5] == constantModule.SENSOR_LASER:
            _laser_cnt = _laser_cnt + 1
            print("|" + " " * 2 + "LASER" + " " * 4 + "|" + " " * 6, _laser_cnt, " " * 6 + "|")
            print("+" + "-" * 11 + "+" + "-" * 15 + "+")
            if sensor == constantModule.SENSOR_LASER:
                _stsensor = 'LASER'
        elif df._regedit[_cnt][5] == constantModule.SENSOR_GEOMAGNETIC:
            _geomagnetic_cnt = _geomagnetic_cnt + 1
            print("|" + "GEOMAGNETIC" + "|" + " " * 6, _geomagnetic_cnt, " " * 6 + "|")
            print("+" + "-" * 11 + "+" + "-" * 15 + "+")
            if sensor == constantModule.SENSOR_GEOMAGNETIC:
                _stsensor = 'GEOMAGNETIC'
        # elif df._regedit[_cnt][5] == constantModule.SENSOR_FLAME:
        #     _flame_cnt = _flame_cnt + 1
        #     print("|" + " " * 2 + "FLAME" + " " + "|" + " " * 6, _flame_cnt, " " * 6 + "|")
        #     print("+" + "-" * 11 + "+" + "-" * 15 + "+")
        #     if sensor == constantModule.SENSOR_FLAME:
        #         _stsensor = 'FLAME'
        elif df._regedit[_cnt][5] == constantModule.SENSOR_ATTITUDE:
            _attitude_cnt = _attitude_cnt + 1
            print("|" + " " * 2 + "ATTITUDE" + " " + "|" + " " * 6, _attitude_cnt, " " * 6 + "|")
            print("+" + "-" * 11 + "+" + "-" * 15 + "+")
            if sensor == constantModule.SENSOR_ATTITUDE:
                _stsensor = 'ATTITUDE'
        elif df._regedit[_cnt][5] == constantModule.SENSOR_RGBLIGHT:
            _rgblight_cnt = _rgblight_cnt + 1
            print("|" + " " * 2 + "RGBLIGHT" + " " + "|" + " " * 6, _rgblight_cnt, " " * 6 + "|")
            print("+" + "-" * 11 + "+" + "-" * 15 + "+")
            if sensor == constantModule.SENSOR_RGBLIGHT:
                _stsensor = 'RGBLIGHT'
        elif df._regedit[_cnt][5] == constantModule.SENSOR_BUZZER:
            _buzzer_cnt = _buzzer_cnt + 1
            print("|" + " " * 2 + "BUZZER" + " " + "|" + " " * 6, _buzzer_cnt, " " * 6 + "|")
            print("+" + "-" * 11 + "+" + "-" * 15 + "+")
            if sensor == constantModule.SENSOR_BUZZER:
                _stsensor = 'BUZZER'
        elif df._regedit[_cnt][5] == constantModule.SENSOR_PHOTOSENSITIVE:
            _sensitive_cnt = _sensitive_cnt + 1
            print("|" + " " * 2 + "SENSITIVE" + " " + "|" + " " * 6, _sensitive_cnt, " " * 6 + "|")
            print("+" + "-" * 11 + "+" + "-" * 15 + "+")
            if sensor == constantModule.SENSOR_PHOTOSENSITIVE:
                _stsensor = 'SENSITIVE'

    print("Looking for the ", ID, " " * 2, _stsensor, '......')

    _sensorID = 0
    _cnt = 0
    _index = 0
    _regedit_num = 0
    for _cnt in list(range(df._regeditNum)):
        _index = _index + 1
        if sensor == df._regedit[_cnt][5]:
            _sensorID = _sensorID + 1
            if _sensorID < ID:
                continue
            elif _sensorID == ID:
                _regedit_num = df._regedit[_cnt][4]
                _protocol[4] = _regedit_num
                _protocol[5] = 0
                _protocol[7] = 1
                xor = df._data_xor_check(_protocol[0:-1])
                _protocol[-2] = xor
                loop = asyncio.get_event_loop()
                loop.run_until_complete(df.sendDate(_protocol, df.ONEBOT))
                # df._ONEBOT.receiveData()
                # df._protocol_analysis()
                break


